;; This Source Code Form is subject to the terms of the Mozilla Public
;; License, v. 2.0. If a copy of the MPL was not distributed with this
;; file, You can obtain one at http://mozilla.org/MPL/2.0/.
;;
;; Copyright (c) 2016 Andrey Antukh <niwi@niwi.nz>

(ns uxbox.main.ui.shapes.icon
  (:require
   [rumext.alpha :as mf]
   [cuerdas.core :as str]
   [uxbox.util.geom.shapes :as geom]
   [uxbox.main.refs :as refs]
   [uxbox.main.ui.shapes.attrs :as attrs]
   [uxbox.main.ui.shapes.common :as common]
   [uxbox.util.interop :as itr]))


;; --- Icon Wrapper

(declare icon-shape)

(mf/defc icon-wrapper
  [{:keys [shape frame] :as props}]
  (let [selected (mf/deref refs/selected-shapes)
        selected? (contains? selected (:id shape))
        on-mouse-down #(common/on-mouse-down % shape)]
    [:g.shape {:class (when selected? "selected")
               :on-mouse-down on-mouse-down}
     [:& icon-shape {:shape (geom/transform-shape frame shape)}]]))

;; --- Icon Shape

(mf/defc icon-shape
  [{:keys [shape] :as props}]
  (let [{:keys [id x y width height metadata rotation content] :as shape} shape
        transform (when (and rotation (pos? rotation))
                    (str/format "rotate(%s %s %s)"
                                rotation
                                (+ x (/ width 2))
                                (+ y (/ height 2))))

        view-box (apply str (interpose " " (:view-box metadata)))

        props (-> (attrs/extract-style-attrs shape)
                  (itr/obj-assign!
                   #js {:x x
                        :y y
                        :transform transform
                        :id (str "shape-" id)
                        :width width
                        :height height
                        :viewBox view-box
                        :preserveAspectRatio "none"
                        :dangerouslySetInnerHTML #js {:__html content}}))]


    [:g {:transform transform}
     [:> "svg" props]]))

;; --- Icon SVG

(mf/defc icon-svg
  [{:keys [shape] :as props}]
  (let [{:keys [content id metadata]} shape
        view-box (apply str (interpose " " (:view-box metadata)))
        props {:viewBox view-box
               :id (str "shape-" id)
               :dangerouslySetInnerHTML #js {:__html content}}]
    [:& "svg" props]))
